

def is_valid_ip(ip):
        try:
            parts = ip.split('.')
            if len(parts) != 4:
                return False
            for part in parts:
                if not 0 <= int(part) <= 255:
                    return False
            return True
        except ValueError:
            return False


def calculate_ip_class(ip):
        """ Funtion to calculate the class of the ipaddress """
        first_octet = int(ip.split('.')[0])
        if 1 <= first_octet <= 126:
            return "A"
        elif 128 <= first_octet <= 191:
            return "B"
        elif 192 <= first_octet <= 223:
            return "C"
        elif 224 <= first_octet <= 239:
            return "D (Multicast)"
        elif 240 <= first_octet <= 255:
            return "E (Experimental)"
        else:
            return "Invalid"


def calculate_subnet_mask(ip):
        """Function to calculate the subnet mask of the a ipadress"""
        if not is_valid_ip(ip):
            return "Invalid IP address."
        
        first_octet = int(ip.split('.')[0])
        
        if 1 <= first_octet <= 126:
            return "255.0.0.0"
        elif 128 <= first_octet <= 191:
            return "255.255.0.0"
        elif 192 <= first_octet <= 223:
            return "255.255.255.0"
        else:
            return "Subnet mask not applicable."
        
def calculate_ip_count(ip):
    first_octet = int(ip.split('.')[0])
    
    if 1 <= first_octet <= 126:
        return 2 ** 24  # Classe A (16.777.214 endereços IP para hosts)
    elif 128 <= first_octet <= 191:
        return 2 ** 16  # Classe B (65.534 endereços IP para hosts)
    elif 192 <= first_octet <= 223:
        return 2 ** 8   # Classe C (254 endereços IP para hosts)
    else:
        return "Não aplicável"
    

def calculate_subnet_notation(subnet_notation):
    try:
        ip, subnet_mask = subnet_notation.split("/")
        ip_parts = ip.split(".")
        subnet_bits = int(subnet_mask)

        if 0 <= subnet_bits <= 32 and len(ip_parts) == 4:
            ip_count = 2 ** (32 - subnet_bits)
            return ip_count
        else:
            return "Notação de sub-rede inválida"
    except ValueError:
        return "Notação de sub-rede inválida"


def get_ip_class(ip):
    first_octet = int(ip.split('.')[0])
    if 1 <= first_octet <= 126:
        return 'A'
    elif 128 <= first_octet <= 191:
        return 'B'
    elif 192 <= first_octet <= 223:
        return 'C'
    elif 224 <= first_octet <= 239:
        return 'D (Multicast)'
    elif 240 <= first_octet <= 255:
        return 'E (Experimental)'
    else:
        return 'Inválido'

# Função para calcular as notações de sub-rede possíveis com base na classe
def calculate_possible_subnets(ip_class):
    if ip_class == 'A':
        return ['/8', '/16', '/24']
    elif ip_class == 'B':
        return ['/16', '/24']
    elif ip_class == 'C':
        return ['/24']
    else:
        return []



import argparse
from direnumerate.__main__ import DirScan, PortScan, FindPattern, InfoIp
from direnumerate.colors import Color
from direnumerate.banner import *
from direnumerate.version import __version__


def dir_scan(args):
    """
    Perform directory enumeration based on the provided arguments.

    Args:
        args (argparse.Namespace): Command-line arguments and options.
    """
    show_banner()

    if args.verbose:
            try:
                url = args.target
                wordlist_file = args.wordlist

                enum = DirScan(url)
                enum.dir_enum(wordlist_file, verbose=True)
            except TypeError:
                print(Color.GREEN + "-------------------- Scan Finished --------------------" + Color.RESET)
            except KeyboardInterrupt:
                print(Color.GREEN + "-------------- Attempt interrupted by user ------------" + Color.RESET)
    else:
        pass

    try:
        url = args.target
        wordlist_file = args.wordlist

        enum = DirScan(url)
        enum.dir_enum(wordlist_file)
    except TypeError:
        print(Color.GREEN + "-------------------- Scan Finished --------------------" + Color.RESET)
    except KeyboardInterrupt:
        print(Color.GREEN + "-------------- Attempt interrupted by user ------------" + Color.RESET)


def port_scan(args):
    """
    Perform port scanning based on the provided arguments.

    Args:
        args (argparse.Namespace): Command-line arguments and options.
    """
    show_banner()
    try:
        host = args.target
        ports = args.ports

        scanner = PortScan(host)
        scanner.scan_ports(ports)
    except KeyboardInterrupt:
        print(Color.GREEN + "-------------- Port scan interrupted by user ------------" + Color.RESET)


def find_pattern(args):
    show_banner()
    file_name_log = args.logname
    key = args.keyword

    fp = FindPattern(file_name_log)
    fp.find_in_log(keyword=key)


def show_info_ip(args):
    show_banner()
    ip_address = args.target

    ipinfo = InfoIp(ip_address)
    ipinfo.show_info()


def main():
    """
    The main function for the Direnumerate application.

    Parses command-line arguments and options, and executes the appropriate function.
    """
    parser = argparse.ArgumentParser(description="Direnumerate - Directory Enumeration on Web Servers")
    
    # Add arguments for directory enumeration
    parser.add_argument("-v", "--verbose", required=False, action="store_true", help="Verbose output")
    parser.add_argument("-t", "--target", required=False, help="Target URL (including scheme, e.g. http://www.example.com)")
    parser.add_argument("-w", "--wordlist", required=False, help="Wordlist file")
    parser.add_argument("-i", "--info", required=False, help="Target host for info scanning")
    
    # Add arguments for port scanning
    parser.add_argument("-p", "--ports", nargs='+', type=int, help="Ports to scan (e.g., 22 80 443)")
    
    # Add arguments for log scanning
    parser.add_argument("-log", "--logname", help="Log Name")
    parser.add_argument("-key", "--keyword", help="Key Word")
    

    args = parser.parse_args()
    
    if args.logname and args.keyword:
        find_pattern(args)
    elif args.ports:
        port_scan(args)
    elif args.info:
        show_info_ip(args)
    else:
        dir_scan(args)

if __name__ == "__main__":
    main()



def create_wordlist(filename):
    """
    Create a wordlist file with commonly used directory and file names.

    This function generates a wordlist containing common directory and file names used for directory enumeration.
    The wordlist is written to the specified filename.

    Args:
        filename (str): The name of the wordlist file to create.

    Example:
        create_wordlist("wordlist.txt")
    """
    wordlist = [
        "a",
        "b",
        "c",
        "d",
        "e",
        "f",
        "g",
        "h",
        "i",
        "j",
        "k",
        "l",
        "m",
        "n",
        "o",
        "p",
        "q",
        "r",
        "s",
        "t",
        "u",
        "v",
        "w",
        "x",
        "y",
        "z",
        "admin",
        "login",
        "images",
        "js",
        "css",
        "config",
        "index",
        "robots.txt",
        "backup",
        "secret",
        "support",
        "contact",
        "blog",
        "forum",
        "product",
        "category",
        "brand",
        "password",
        "123456",
        "qwerty",
        "letmein",
        "access",
        "admin123",
        "administrator",
        "superuser",
        "root",
        "test",
        # PHP apps
        "config/config.php",
        "includes/config.inc",
        
        # PHP apps
        "guest",
        "demo",
        "webmaster",
        "developer",
        "api",
        "api_key",
        "db",
        "database",
        "user",
        "users",
        "auth",
        "authentication",
        "login.php",
        "wp-admin",
        "wp-login.php",
        "upload",
        "uploads",
        "download",
        "downloads",
        "private",
        "public",
        "static",
        "media",
        "files",
        "file",
        "images/",
        "js/",
        "css/",
        "assets/",
        "images/uploads",
        "userfiles",
        "data",
        "temp",
        "temp/",
        "logs",
        "scripts",
        "web",
        "public_html",
        "secure",
        "private/",
        "backup/",
        "backup_files/",
        "conf",
        "config/",
        "configurations/",
        "install",
        "installer",
        "setup",
        "tmp",
        "db.sql",
        "data.sql",
        "users.sql",
        "passwords.txt",
        "passwd",
        "pass",
        "password.txt",
        "login.txt",
        "credentials",
        "phpmyadmin",
        "phpMyAdmin",
        "myadmin",
        "myAdmin",
        "panel",
        "control",
        "manager",
        "admin-panel",
        "admin-login",
        "signin",
        "sign-in",
        "login-form",
        "login.html",
        "login.aspx",
        "login.jsp",
        "signin.html",
        "signin.aspx",
        "signin.jsp",
        "register",
        "signup",
        "register.html",
        "register.aspx",
        "signup.html",
        "signup.aspx",
        "forgot-password",
        "forgot-password.html",
        "forgot-password.aspx",
        "reset-password",
        "reset-password.html",
        "reset-password.aspx",
        "contact-us",
        "contact.html",
        "contact.aspx",
        "contact-form",
        "support.html",
        "support.aspx",
        "support-form",
        "about-us",
        "about.html",
        "about.aspx",
        "faq",
        "faq.html",
        "faq.aspx",
        "terms",
        "terms.html",
        "terms.aspx",
        "privacy",
        "privacy.html",
        "privacy.aspx",
        "legal",
        "legal.html",
        "legal.aspx",
        "policy",
        "policy.html",
        "policy.aspx",
        "blog/",
        "news",
        "news.html",
        "news.aspx",
        "articles",
        "articles.html",
        "articles.aspx",
        "events",
        "events.html",
        "events.aspx",
        "products",
        "products.html",
        "products.aspx",
        "services",
        "services.html",
        "services.aspx",
        "portfolio",
        "portfolio.html",
        "portfolio.aspx",
        "gallery",
        "gallery.html",
        "gallery.aspx",
        "download/",
        "downloads/",
        "download-files",
        "download-file",
        "download.html",
        "download.aspx",
        "download.jsp",
        "upload/",
        "uploads/",
        "upload-file",
        "upload.html",
        "upload.aspx",
        "upload.jsp",
        "logout",
        "logout.html",
        "logout.aspx",
        "logout.jsp",
        "signout",
        "signout.html",
        "signout.aspx",
        "signout.jsp",
        "clients",
        "client",
        "adm",
        "db.sqlite",
        "db.sqlite3",
        "public_html/admin",
        "useruploads",
        "tempfiles",
        "logs/",
        "scripts/",
        "www",
        "htdocs",
        "site",
        "system",
        "platform",
        "app",
        "data/",
        "users/",
        "database/",
        "storage",
        "cache",
        "assets/",
        "lib",
        "include",
        "html",
        "htm",
        "php",
        "asp",
        "jsp",
        "exe",
        "bin",
        "dev",
        "config.php",
        "config.inc",
        "config.ini",
        "settings.php",
        "settings.inc",
        "settings.ini",
        "photo",
        "video",
        "hd",
        "HD",
        "home.php",
        "wp-content",
        "img/users/full/",
        "category",
        "index.php",
        "index.html",
        "index.jsp",
        "index.aspx",
        "index.htm",
        "of",
        "of/",
        "of.html",
        "of.aspx",
        "of.jsp",
        "index-of",
        "file",
        "files/",
        "files",
        "directory",
        "dir",
        "dir/",
        "dir.html",
        "dir.aspx",
        "dir.jsp",
        "list",
        "list.html",
        "list.aspx",
        "list.jsp",
        "view",
        "view.html",
        "view.aspx",
        "view.jsp",
        "explore",
        "explore.html",
        "explore.aspx",
        "explore.jsp",
        "browse",
        "browse.html",
        "browse.aspx",
        "browse.jsp",
        "content",
        "content.html",
        "content.aspx",
        "content.jsp",
        "show",
        "show.html",
        "show.aspx",
        "show.jsp",
        "display",
        "display.html",
        "display.aspx",
        "display.jsp",
        "fetch",
        "fetch.html",
        "fetch.aspx",
        "fetch.jsp",
        "get",
        "get.html",
        "get.aspx",
        "get.jsp",
        "retrieve",
        "retrieve.html",
        "retrieve.aspx",
        "retrieve.jsp",
        "access",
        "access.html",
        "access.aspx",
        "access.jsp",
        "open",
        "open.html",
        "open.aspx",
        "open.jsp",
        "read",
        "read.html",
        "read.aspx",
        "read.jsp",
        "logs",
        "logfiles",
        "access_logs",
        "error_logs",
        "security_logs",
        "event_logs",
        "audit_logs",
        "syslogs",
        "app_logs",
        "debug_logs",
        "traffic_logs",
        "server_logs",
        "apache_logs",
        "nginx_logs",
        "iis_logs",
        "web_logs",
        "firewall_logs",
        "proxy_logs",
        "database_logs",
        "application_logs",
        "error_log.txt",
        "access_log.txt",
        "security_log.txt",
        "event_log.txt",
        "audit_log.txt",
        "syslog.txt",
        "app_log.txt",
        "debug_log.txt",
        "traffic_log.txt",
        "server_log.txt",
        "apache_log.txt",
        "nginx_log.txt",
        "iis_log.txt",
        "web_log.txt",
        "firewall_log.txt",
        "proxy_log.txt",
        "database_log.txt",
        "application_log.txt",
        "log_backup",
        "log_archives",
        "log_history",
        "log_data",
        "log_files",
        "log_records",
        "log_entries",
        "log_storage",
        "log_management",
        "log_viewer",
        "log_analyzer",
        "log_parser",
        "log_monitoring",
        "log_security",
        "log_alerts",
        "log_analysis",
        "log_search",
        "log_export",
        "log_rotation",
        "log_cleanup",
        "log_rotation",
        "log_rotation_script",
        "log_rotation_config",
        "log_purge",
        "log_alert",
        "log_notifications",
        "log_errors",
        "log_warnings",
        "log_info",
        "log_debug",
        "log_critical",
        "log_alert",
        "log_emergency",
        "log_severity",
        "log_level",
        "log_timestamp",
        "log_date",
        "log_time",
        "log_ip",
        "log_user",
        "log_source",
        "log_destination",
        "log_event",
        "log_message",
        "log_code",
        "log_identifier",
        "log_category",
        "log_type",
        "log_origin",
        "log_target",
        "log_client",
        "log_server",
        "log_protocol",
        "log_response",
        "log_request",
        "log_access",
        "log_authentication",
        "log_authorization",
        "log_permission",
        "log_denied",
        "log_granted",
        "log_success",
        "log_failure",
        "log_login",
        "log_logout",
        "log_session",
        "log_session_id",
        "log_user_agent",
        "log_http",
        "log_https",
        "log_ftp",
        "log_ssh",
        "log_telnet",
        "log_smtp",
        "log_pop3",
        "log_imap",
        "log_dns",
        "log_database",
        "log_app",
        "log_firewall",
        "log_proxy",
        "log_vpn",
        "log_virus",
        "log_malware",
        "log_intrusion",
        "log_attack",
        "log_ddos",
        "log_security_event",
        "log_critical_event",
        "log_alert_event",
        "log_error_event",
        "log_warning_event",
        "log_info_event",
        "log_debug_event",
        "log_event_id",
        "log_event_type",
        "log_event_description",
        "log_event_details",
        "log_event_context",
        "log_event_origin",
        "log_event_destination",
        "log_event_source_ip",
        "log_event_target_ip",
        "log_event_timestamp",
        "log_event_date",
        "log_event_time",
        "log_event_user",
        "log_event_category",
        "log_event_severity",
        "log_event_level",
        "log_event_status",
        "log_event_response",
        "log_event_request",
        "log_event_access",
        "log_event_authentication",
        "log_event_authorization",
        "log_event_permission",
        "log_event_denied",
        "log_event_granted",
        "log_event_success",
        "log_event_failure",
        "log_event_login",
        "log_event_logout",
        "log_event_session",
        "log_event_session_id",
        "log_event_user_agent",
        "log_event_http",
        "log_event_https",
        "log_event_ftp",
        "log_event_ssh",
        "log_event_telnet",
        "log_event_smtp",
        "log_event_pop3",
        "log_event_imap",
        "log_event_dns",
        "log_event_database",
        "log_event_app",
        "log_event_firewall",
        "log_event_proxy",
        "log_event_vpn",
        "log_event_virus",
        "log_event_malware",
        "log_event_intrusion",
        "log_event_attack",
        "log_event_ddos",
        "log_event_security_alert",
        "log_event_critical_alert",
        "log_event_warning_alert",
        "log_event_info_alert",
        "log_event_debug_alert",
        "log_event_threat",
        "log_event_vulnerability",
        "log_event_incident",
        "log_event_response_time",
        "log_event_server_error",
        "log_event_client_error",
        "log_event_http_status",
        "log_event_http_code",
        "log_event_http_response",
        "log_event_http_request",
        "log_event_api",
        "log_event_configuration",
        "log_event_settings",
        "log_event_changes",
        "log_event_modification",
        "log_event_audit",
        "log_event_compliance",
        "log_event_policy",
        "log_event_rules",
        "log_event_user_activity",
        "log_event_admin_activity",
        "log_event_system_activity",
        "log_event_network_activity",
        "log_event_server_activity",
        "log_event_application_activity",
        "log_event_database_activity",
        "log_event_traffic",
        "log_event_bandwidth",
        "log_event_latency",
        "log_event_error_rate",
        "log_event_warning_rate",
        "log_event_info_rate",
        "log_event_debug_rate",
        "log_event_security_rate",
        "log_event_access_rate",
        "log_event_authentication_rate",
        "log_event_authorization_rate",
        "log_event_permission_rate",
        "log_event_denied_rate",
        "log_event_granted_rate",
        "log_event_success_rate",
        "log_event_failure_rate",
        "log_event_login_rate",
        "log_event_logout_rate",
        "log_event_session_rate",
        "log_event_session_id_rate",
        "log_event_user_agent_rate",
        "log_event_http_rate",
        "log_event_https_rate",
        "log_event_ftp_rate",
        "log_event_ssh_rate",
        "log_event_telnet_rate",
        "log_event_smtp_rate",
        "log_event_pop3_rate",
        "log_event_imap_rate",
        "log_event_dns_rate",
        "log_event_database_rate",
        "log_event_app_rate",
        "log_event_firewall_rate",
        "log_event_proxy",
        "sitemap",
        "sitemap.xml",
        "site-map",
        "site_map",
        "site-map.xml",
        "site_map.xml",
        "sitemap_html",
        "sitemap_html.xml",
        "sitemap_index",
        "sitemap_index.xml",
        "sitemap_index_html",
        "sitemap_index_html.xml",
        "sitemap_index_html.txt",
        "sitemap_index_html.gz",
        "sitemap_index_html.zip",
        "sitemap_index_html.tar",
        "sitemap_index_html.tar.gz",
        "sitemap_index_html.tar.bz2",
        "sitemap_index_html.7z",
        "sitemap_index_html.rar",
        "sitemap_index_html.tar.xz",
        "sitemap_index_html.tbz2",
        "sitemap_index_html.tbz",
        "sitemap_index_html.cpio",
        "sitemap_index_html.lz",
        "sitemap_index_html.lzma",
        "sitemap_index_html.Z",
        "sitemap_index_html.bz2",
        "sitemap_index_html.arj",
        "sitemap_index_html.deb",
        "sitemap_index_html.rpm",
        "sitemap_index_html.zipx",
        "sitemap_index_html.tgz",
        "sitemap_index_html.xz",
        "sitemap_index_html.cab",
        "sitemap_index_html.ace",
        "sitemap_index_html.wim",
        "sitemap_index_html.dmg",
        "sitemap_index_html.iso",
        "sitemap_index_html.vhd",
        "sitemap_index_html.vhdx",
        "sitemap_index_html.vmdk",
        "sitemap_index_html.udf",
        "sitemap_index_html.fat",
        "sitemap_index_html.ntfs",
        "sitemap_index_html.exfat",
        "sitemap_index_html.hfs",
        "sitemap_index_html.hfs+",
        "sitemap_index_html.btrfs",
        "sitemap_index_html.ext2",
        "sitemap_index_html.ext3",
        "sitemap_index_html.ext4",
        "sitemap_index_html.reiserfs",
        "sitemap_index_html.jfs",
        "sitemap_index_html.xfs",
        "sitemap_index_html.zfs",
        "sitemap_index_html.ufs",
        "sitemap_index_html.minix",
        "sitemap_index_html.apfs",
        "sitemap_index_html.hfsx",
        "sitemap_index_html.qnx",
        "sitemap_index_html.romfs",
        "sitemap_index_html.squashfs",
        "sitemap_index_html.ubifs",
        "sitemap_index_html.yaffs",
        "sitemap_index_html.ecryptfs",
        "sitemap_index_html.f2fs",
        "sitemap_index_html.nilfs",
        "sitemap_index_html.vxfs",
        "sitemap_index_html.gfs",
        "sitemap_index_html.cifs",
        "sitemap_index_html.nfs",
        "sitemap_index_html.afs",
        "sitemap_index_html.9p",
        "sitemap_index_html.npfs",
        "sitemap_index_html.fh",
        "sitemap_index_html.ofs",
        "sitemap_index_html.hpfs",
        "sitemap_index_html.befs",
        "sitemap_index_html.gpfs",
        "sitemap_index_html.wufs",
        "sitemap_index_html.cramfs",
        "sitemap_index_html.romfs",
        "sitemap_index_html.squashfs",
        "sitemap_index_html.ubifs",
        "sitemap_index_html.yaffs",
        "sitemap_index_html.ecryptfs",
        "sitemap_index_html.f2fs",
        "sitemap_index_html.nilfs",
        "sitemap_index_html.vxfs",
        "sitemap_index_html.gfs",
        "sitemap_index_html.cifs",
        "sitemap_index_html.nfs",
        "sitemap_index_html.afs",
        "sitemap_index_html.9p",
        "sitemap_index_html.npfs",
        "sitemap_index_html.fh",
        "sitemap_index_html.ofs",
        "sitemap_index_html.hpfs",
        "sitemap_index_html.befs",
        "sitemap_index_html.gpfs",
        "sitemap_index_html.wufs",
        "sitemap_index_html.cramfs",
        "sitemap_index_html.efs",
        "sitemap_index_html.ntfs3g",
        "sitemap_index_html.ufs",
        "sitemap_index_html.romfs",
        "sitemap_index_html.squashfs",
        "sitemap_index_html.ubifs",
        "sitemap_index_html.yaffs",
        "sitemap_index_html.ecryptfs",
        "sitemap_index_html.f2fs",
        "sitemap_index_html.nilfs",
        "sitemap_index_html.vxfs",
        "sitemap_index_html.gfs",
        "sitemap_index_html.cifs",
        "sitemap_index_html.nfs",
        "sitemap_index_html.afs",
        "sitemap_index_html.9p",
        "sitemap_index_html.npfs",
        "sitemap_index_html.fh",
        "sitemap_index_html.ofs",
        "sitemap_index_html.hpfs",
        "sitemap_index_html.befs",
        "sitemap_index_html.gpfs",
        "sitemap_index_html.wufs",
        "sitemap_index_html.cramfs",
        "sitemap_index_html.efs",
        "sitemap_index_html.ntfs3g",
        "sitemap_index_html.ufs",
        "sitemap_index_html.romfs",
        "sitemap_index_html.squashfs",
        "sitemap_index_html.ubifs",
        "sitemap_index_html.yaffs",
        "sitemap_index_html.ecryptfs",
        "sitemap_index_html.f2fs",
        "sitemap_index_html.nilfs",
        "sitemap_index_html.vxfs",
        "sitemap_index_html.gfs",
        "sitemap_index_html.cifs",
        "sitemap_index_html.nfs",
        "sitemap_index_html.afs",
        "sitemap_index_html.9p",
        "sitemap_index_html.npfs",
        "sitemap_index_html.fh",
        "sitemap_index_html.ofs",
        "sitemap_index_html.hpfs",
        "sitemap_index_html.befs",
        "sitemap_index_html.gpfs",
        "sitemap_index_html.wufs",
        "sitemap_index_html.cramfs",
        "sitemap_index_html.efs",
        "sitemap_index_html.ntfs3g",
        "sitemap_index_html.ufs",
        "sitemap_index_html.romfs",
        "sitemap_index_html.squashfs",
        "sitemap_index_html.ubifs",
        "sitemap_index_html.yaffs",
        "sitemap_index_html.ecryptfs",
        "sitemap_index_html.f2fs",
        "sitemap_index_html.nilfs",
        "sitemap_index_html.vxfs",
        "sitemap_index_html.gfs",
        "sitemap_index_html.cifs",
        "sitemap_index_html.nfs",
        "sitemap_index_html.afs",
        "sitemap_index_html.9p",
        "sitemap_index_html.npfs",
        "sitemap_index_html.fh",
        "sitemap_index_html.ofs",
        "sitemap_index_html.hpfs",
        "sitemap_index_html.befs",
        "sitemap_index_html.gpfs",
        "sitemap_index_html.wufs",
        "sitemap_index_html.cramfs",
        "sitemap_index_html.efs",
        "sitemap_index_html.ntfs3g",
        "sitemap_index_html.ufs",
        "sitemap_index_html.romfs",
        "sitemap_index_html.squashfs",
        "sitemap_index_html.ubifs",
        "sitemap_index_html.yaffs",
        "admin/user",
        "admin/pictures",
        "admin/files",
        "admin/docs",
        "admin/logs",
        "user/admin",
        "user/pictures",
        "user/files",
        "user/docs",
        "user/logs",
        "pictures/admin",
        "pictures/user",
        "pictures/files",
        "pictures/docs",
        "pictures/logs",
        "files/admin",
        "files/user",
        "files/pictures",
        "files/docs",
        "files/logs",
        "docs/admin",
        "docs/user",
        "docs/pictures",
        "docs/files",
        "docs/logs",
        "logs/admin",
        "logs/user",
        "logs/pictures",
        "logs/files",
        "logs/docs",
        "admin/user/pictures",
        "admin/user/files",
        "admin/user/docs",
        "admin/user/logs",
        "admin/pictures/files",
        "admin/pictures/docs",
        "admin/pictures/logs",
        "admin/files/docs",
        "admin/files/logs",
        "admin/docs/logs",
        "user/admin/pictures",
        "user/admin/files",
        "user/admin/docs",
        "user/admin/logs",
        "user/pictures/files",
        "user/pictures/docs",
        "user/pictures/logs",
        "user/files/docs",
        "user/files/logs",
        "user/docs/logs",
        "pictures/admin/user",
        "pictures/admin/files",
        "pictures/admin/docs",
        "pictures/admin/logs",
        "pictures/user/files",
        "pictures/user/docs",
        "pictures/user/logs",
        "pictures/files/docs",
        "pictures/files/logs",
        "pictures/docs/logs",
        "admin/user/pictures/files",
        "admin/user/pictures/docs",
        "admin/user/pictures/logs",
        "admin/user/files/docs",
        "admin/user/files/logs",
        "admin/user/docs/logs",
        "admin/pictures/files/docs",
        "admin/pictures/files/logs",
        "admin/pictures/docs/logs",
        "user/pictures/files/docs",
        "user/pictures/files/logs",
        "user/pictures/docs/logs",
        "admin/user/pictures/files/docs",
        "admin/user/pictures/files/logs",
        "admin/user/pictures/docs/logs",
        "admin/pictures/files/docs/logs",
        "user/pictures/files/docs/logs",
        "admin/user/pictures/files/docs/logs",
        "/user/pictures/files/",
        "/user/pictures/",
        "admin/docs",
        "user/docs",
        "pictures/docs",
        "files/docs",
        "logs/docs",
        "admin/logs",
        "user/logs",
        "pictures/logs",
        "files/logs",
        "admin/user/pictures",
        "admin/user/files",
        "admin/user/docs",
        "admin/user/logs",
        "admin/pictures/files",
        "admin/pictures/docs",
        "admin/pictures/logs",
        "admin/files/docs",
        "admin/files/logs",
        "admin/docs/logs",
        "user/pictures/files",
        "user/pictures/docs",
        "user/pictures/logs",
        "user/files/docs",
        "user/files/logs",
        "user/docs/logs",
        "pictures/files/docs",
        "pictures/files/logs",
        "pictures/docs/logs",
        "/user/pictures/files",
        "/user/pictures/docs",
        "/user/pictures/logs",
        "/user/files/docs",
        "/user/files/logs",
        "/img/members/",
        "img",
        ]

    with open(filename, "w") as wordlist_file:
        for word in wordlist:
            wordlist_file.write(word + "\n")



# this is part of the Direnumerate project.
#
# Copyright ©  2023 - 2024 Juan Bindez  <juanbindez780@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import requests
import socket
import os
import webbrowser
from typing import Optional

from direnumerate.createlist import create_wordlist
from direnumerate.colors import Color
from direnumerate.banner import *
from direnumerate.getinfo import get_info_ip
from direnumerate.ipcalculator import *
from direnumerate.list_urls_accounts import *


class DirScan:
    """
    A class for directory scanning.

    Attributes:
        url (str): The URL to scan.
        wordlist_file (str): The path to the wordlist file.
    """
    def __init__(self, url):
        """
        Initializes a DirScan instance.

        Args:
            url (str): The URL to scan.
            wordlist_file (str): The path to the wordlist file.
        """
        self.url = "http://" + url.replace("https://", "")

    def dir_enum(self, wordlist_file, verbose: bool = False):
        """
        Perform directory enumeration.

        This method reads paths from a wordlist file, appends them to the URL, and sends HTTP requests to
        check for the existence of the resources. It prints the results based on the response status code.

        - 200: Found
        - 204: No Content
        - 400: Bad Request
        - 401: Unauthorized
        - 403: Forbidden
        - 404: Not Found
        - 500: Internal Server Error

        If the wordlist file is not found, it attempts to create one.
        """
        self.wordlist_file = wordlist_file
        

        if verbose:
            try:
                with open(self.wordlist_file, "r") as self.wordlist_file:
                    for line in self.wordlist_file:
                        path = line.strip()
                        full_url = self.url + "/" + path
                        response = requests.get(full_url)
                        
                        if response.status_code == 200:
                            results = f"{Color.GREEN}[Found]:{Color.RESET} {full_url}"
                            print(results)
                            
                        elif response.status_code == 204:
                            results = f"{Color.BLUE}[No Content]:{Color.RESET} {full_url}"
                            print(results)
                            
                        elif response.status_code == 400:
                            results = f"{Color.YELLOW}[Bad Request]:{Color.RESET} {full_url}"
                            print(results)
                            
                        elif response.status_code == 401:
                            results = f"{Color.RED}[Unauthorized]:{Color.RESET} {full_url}"
                            print(results)
                            
                        elif response.status_code == 403:
                            results = f"{Color.RED}[Forbidden]:{Color.RESET} {full_url}"
                            print(results)
                            
                        elif response.status_code == 404:
                            results =f"{Color.YELLOW}[Not Found]:{Color.RESET} {full_url}"
                            print(results)
                            
                        elif response.status_code == 500:
                            results = f"{Color.BLUE}[Internal Server Error]:{Color.RESET} {full_url}"
                            print(results)
                            

            except FileNotFoundError:
                print(Color.RED + "Word list file not found." + Color.RESET)
                
            except TypeError:
                print(Color.GREEN + "-------------------- Scan Finished --------------------" + Color.RESET)
                
            except KeyboardInterrupt:
                print(Color.GREEN + "-------------- Attempt interrupted by user ------------" + Color.RESET)

            except requests.exceptions.ConnectionError as rec:
                print(rec)
                print(Color.RED + "[Error] " + Color.RESET)

        else:
            try:
                with open(self.wordlist_file, "r") as self.wordlist_file:
                        for line in self.wordlist_file:
                            path = line.strip()
                            full_url = self.url + "/" + path
                            response = requests.get(full_url)
                            
                            if response.status_code == 200:
                                results = f"{Color.GREEN}[Found]:{Color.RESET} {full_url}"
                                print(results)
            
            except FileNotFoundError:
                print(Color.RED + "Word list file not found." + Color.RESET)
                
            except TypeError:
                print(Color.GREEN + "-------------------- Scan Finished --------------------" + Color.RESET)
                
            except KeyboardInterrupt:
                print(Color.GREEN + "-------------- Attempt interrupted by user ------------" + Color.RESET)

            except requests.exceptions.ConnectionError as rec:
                print(rec)
                print(Color.RED + "[Error]" + Color.RESET)

class PortScan:
    """
    A class for port scanning.

    Attributes:
        host (str): The host to scan for open ports.
        ports (list): A list of ports to scan.
        open_ports (list): A list to store open ports.
    """
    def __init__(self, host):
        """
        Initializes a PortScan instance.

        Args:
            host (str): The host to scan for open ports.
            ports (list): A list of ports to scan.
        """
        self.host = host.replace("https://", "")
        self.open_ports = []

    def scan_ports(self, ports):
        """
        Scan the specified ports on the target host.

        This method attempts to establish a TCP connection to each port in the list of ports on the target host.
        If the connection is successful (result == 0), the port is considered open, and it is added to the open_ports list.
        If the connection fails, the port is considered closed.

        The results are printed to the console, indicating whether each port is open or closed.

        Raises:
            socket.gaierror: If an invalid host is provided.
        """
        self.ports = ports
        
        try:
            for port in self.ports:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock.settimeout(1)

                result = sock.connect_ex((self.host, port))

                if result == 0:
                    self.open_ports.append(port)
                    results = f"{Color.GREEN} [http://{self.host}] port: {port} is open, {Color.RESET}"
                    print(results)
                   
                else:
                    results = f"{Color.RED} [http://{self.host}] port: {port} is closed, {Color.RESET}"
                    print(results)
                
        except socket.gaierror as sq:
            print(sq)
            print(Color.RED + "[Error]" + Color.RESET)
        sock.close()

class FindPattern:
    """
    A class for searching for a keyword in a log file.

    Attributes:
        file_log (str): The path to the log file.
        keyword (str): The keyword to search for in the log file.
    """

    def __init__(self, file_log):
        """
        Initializes a LogScan instance.

        Args:
            file_log (str): The path to the log file.
            keyword (str, optional): The keyword to search for in the log file.
        """
        self.file_log = file_log

    def find_in_log(self, keyword: str = None):
        """
        Search for the specified keyword in the log file and display matching lines.

        This method reads the lines from the log file and identifies lines that contain the specified keyword.
        It then prints the matching lines along with their line numbers.

        If no matching lines are found, a message is printed indicating that no occurrences were found.
        """
        self.keyword = keyword

        with open(self.file_log, 'r') as file:
            lines = file.readlines()

        matching_lines = [line for line in lines if self.keyword in line]

        if matching_lines:
            
            print(Color.GREEN + f"Lines with the word --> '{self.keyword}':" + Color.RESET)
            for i, line in enumerate(matching_lines, start=1):
                print(f"Line {i}: {line.strip()}")
        else:
            
            print(Color.RED + f"No occurrence of the word --> '{self.keyword}' found." + Color.RESET)


class InfoIp:
    """
    A class for obtaining and analyzing information about an IP address, as well as performing IP-related calculations.

    Args:
        ip (str): The IP address to be analyzed.

    Methods:
        ip_calculator(self, all=False, verify_valid_ip=False, verify_class=False, calc_subnet_mask=False, calc_count=False):
            Perform various IP-related calculations and verifications based on the specified flags.

        show_info(self):
            Display detailed information about the IP address, including its geographical location, hostname, and more.

    Attributes:
        ip (str): The IP address to be analyzed.

    The `ip_calculator` method allows you to perform different IP-related tasks based on the specified flags. You can verify if the IP is valid, calculate its class, subnet mask, or the number of IP addresses in its block.

    The `show_info` method retrieves and displays comprehensive information about the IP address, including its geographical details, hostname, and more.

    Example:
        ip_info = InfoIp("192.168.1.1")
        ip_info.show_info()
        ip_info.ip_calculator(verify_valid_ip=True)

    Note:
        This class relies on external functions (not provided in this code snippet) such as `is_valid_ip`, `calculate_ip_class`,
        `calculate_subnet_mask`, `calculate_ip_count`, and `get_info_ip` for its functionality.

    For detailed information on the individual methods and their parameters, please refer to the method's documentation.
    """
    def __init__(self, ip):
        self.ip = ip


    def ip_calculator(self,
                         all: bool = False,
                         verify_valid_ip: bool = False,
                         verify_class: bool = False,
                         calc_subnet_mask: bool = False,
                         calc_count: bool = False,):
        """
        Perform various IP-related calculations and verifications based on the specified flags.

        Args:
            all (bool): Calculate all available information if set to True.
            verify_valid_ip (bool): Verify if the IP address is valid.
            verify_class (bool): Calculate and display the IP address class.
            calc_subnet_mask (bool): Calculate and display the subnet mask.
            calc_count (bool): Calculate and display the number of IP addresses in the block.

        Returns:
            None

        Note:
            This method relies on external functions (not provided in this code snippet) for its functionality.
        """

        if all:
            if is_valid_ip(self.ip):
                # print(Color.GREEN + "Valid IP address." + Color.RESET)
                ip_class = calculate_ip_class(self.ip)
                print(Color.GREEN + "IP address class:" + Color.RESET, f"{ip_class}")
                subnet_mask = calculate_subnet_mask(self.ip)
                print(Color.GREEN + "Subnet mask:" + Color.RESET, f"{subnet_mask}")
                ip_count = calculate_ip_count(self.ip)
                if ip_count == "Not applicable":
                    print(Color.RED + "Cannot calculate for this address class." + Color.RESET)
                else:
                    print(Color.GREEN + "The number of IP addresses in the block is:" + Color.RESET, f"{ip_count}")

               
            else:
                print(Color.RED + "Invalid IP address. Make sure you use the correct format." + Color.RESET)

        elif verify_valid_ip:

            if is_valid_ip(self.ip):
                print(Color.GREEN + "Valid IP address." + Color.RESET)

        elif verify_class:

            ip_class = calculate_ip_class(self.ip)
            print(Color.GREEN + f"IP address class: ",  + Color.RESET + {ip_class})

        elif calc_subnet_mask:

            subnet_mask = calculate_subnet_mask(self.ip)
            print(Color.GREEN + f"Subnet mask:", + Color.RESET + {subnet_mask})

        elif calc_count:
            if is_valid_ip(self.ip):
                ip_count = calculate_ip_count(self.ip)
                if ip_count == "Not applicable":
                    print(Color.RED + "Cannot calculate for this address class.", + Color.RESET)
                else:
                    print(Color.GREEN + f"The number of IP addresses in the block is:"+ Color.RESET, {ip_count})
            else:
                print(Color.RED + "Invalid IP address. Make sure to use the correct format.", + Color.RESET)
    

    def show_info(self):
        """
        Display detailed information about the IP address, including its geographical location, hostname, and more.

        Args:
            None

        Returns:
            None

        Note:
            This method relies on external functions (not provided in this code snippet) for its functionality.
        """
        
        informations = get_info_ip(self.ip)

        ip_info = {
                "Information about the IP address": self.ip,
                "IP": informations["ip:"],
                "Hostname": informations["hostname"],
                "Location": informations["loc"],
                "City": informations["city"],
                "Region": informations["region"],
                "Country": informations["country"],
                "Internet Service Provider": informations["org"],
                # Additional information (if available)
                "Postal Code": informations.get("postal"),
                "Time Zone": informations.get("timezone"),
                "Coordinates": informations.get("loc"),
                "Company Name": informations.get("company"),
                "ASN (Autonomous System Number)": informations.get("asn"),
                "Network Prefix": informations.get("network"),
                "Network Prefix CIDR": informations.get("cidr"),
                "Connection Type": informations.get("type"),
                "Regional Internet Registry (RIR)": informations.get("region"),
                "IP Block": informations.get("ip_block")
        }
        
        ic = InfoIp(self.ip)
        ic.ip_calculator(all=True)

        return ip_info


class UserScan:
    def __init__(self, user_name):
        self.user_name = user_name

    def found_users(self):
        for user in list_accounts:

            full_url = user + self.user_name
            response = requests.get(full_url)
                        
            if response.status_code == 200:
                print(Color.GREEN + f"User Account [Found]: -> {Color.RESET + full_url}")
            else:
                print(Color.RED + f"User Account [Not Found]: -> {Color.RESET + full_url}")

